import logging
import os


class OneLineExceptionFormatter(logging.Formatter):
    def formatException(self, exc_info):
        result = super().formatException(exc_info)
        return repr(result)

    def format(self, record):
        result = super().format(record)
        if record.exc_text:
            result = result.replace("\n", "")
        return result


handler = logging.StreamHandler()
formatter = OneLineExceptionFormatter(logging.BASIC_FORMAT)
handler.setFormatter(formatter)
log = logging.getLogger("witnet_lib")
log.setLevel(os.environ.get("LOGLEVEL", "INFO"))
log.addHandler(handler)



from setuptools import setup, find_packages

import witnet_lib

def install_deps():
    """Reads requirements.txt and preprocess it
    to be feed into setuptools.
    This is the only possible way (we found)
    how requirements.txt can be reused in setup.py
    using dependencies from private github repositories.
    Links must be appendend by `-{StringWithAtLeastOneNumber}`
    or something like that, so e.g. `-9231` works as well as
    `1.1.0`. This is ignored by the setuptools, but has to be there.
    Returns:
         list of packages and dependency links.
    """
    with open('requirements.txt', 'r') as f:
        packages = f.readlines()
        new_pkgs = []
        for resource in packages:
                new_pkgs.append(resource.strip())
        return new_pkgs

def readme():
        with open('README.md','r') as f:
               text = f.read()
        return text
setup(
        name='witnet_lib',
        version=witnet_lib.__version__,
        packages=find_packages(exclude=("tests",)),
        description='Lightclient implementation library for witnet node',
        author='harsh',
        long_description=readme(),
        long_description_content_type="text/markdown",
        author_email='harshjniitr@gmail.com',
        license='MIT',
        url='https://github.com/harsh-98/witnet_lib',
        keywords='witnet blockchain lightclient proto',
        install_requires=install_deps(),
        classifiers=['Development Status :: 3 - Alpha',
                     'Programming Language :: Python :: 3.4',
                     'Programming Language :: Python :: 3.5',
                     'Programming Language :: Python :: 3.6',
                     'Programming Language :: Python :: 3.7',
                     'License :: OSI Approved :: MIT License'],
        setup_requires=['wheel']
)


# class AttrDict(dict):
#     """
#     >>>o=AttrDict(**{"a":1,"b":{"c":1}})
#     >>>o.a
#     1
#     >>>o.b.c # errors
#     >>>o.b
#     {"c":1}
#     >>>a=AttrDict()
#     >>>a.update({"a":1})
#     >>>a.a
#     1

#     Args:
#         dict ([type]): [description]
#     """

#     def __init__(self, *args, **kwargs):
#         super(AttrDict, self).__init__(*args, **kwargs)
#         self.__dict__ = self


class AttrDict(object):
    """
    >>>o=AttrDict({"a":1,"b":[{"c":1}]})
    >>>o.a
    1
    >>o.b[0].c
    1

    Args:
        object (): inherits the object class
    """

    def __init__(self, d):
        """recursively converts dictionary to object

        Args:
            d (dictionary): object
        """
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
                setattr(self, a, [AttrDict(x) if isinstance(
                    x, dict) else x for x in b])
            else:
                setattr(self, a, AttrDict(b) if isinstance(b, dict) else b)


def resolve_url(url):
    """
    >>>resolve_url("127.0.0.1:21337")
    >>>"127.0.0.1", 21337

    >>>resolve_url("http://127.0.0.1:21337")
    >>>"127.0.0.1", 21337


    Args:
        url (string): {http,ws}://ip:port

    Returns:
        (string,int): ip, port
    """
    url = url.split("://")[-1]
    url = url.strip()
    host, port = url.split(":")
    return host, int(port)
